@@ -1,4 +1,4 @@
-# @(#)$Id: Bob.pm 285 2011-07-11 12:40:49Z pjf $
+# @(#)$Id: Bob.pm 321 2011-11-30 00:01:49Z pjf $
 
 package Bob;
 
@@ -14,17 +14,15 @@ BEGIN {
    $ENV{PATH}   =~ m{ \A /home/sand }mx and whimper 'Stopped Konig';
 }
 
-use version; our $VERSION = qv( sprintf '0.6.%d', q$Rev: 285 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.7.%d', q$Rev: 321 $ =~ /\d+/gmx );
 
 use File::Spec::Functions;
 use Module::Build;
 
 sub new {
-   my ($class, $params) = @_;
+   my ($class, $params) = @_; $params ||= {};
 
-   ($params and ref $params eq q(HASH)) or whimper 'No params hash';
-
-   my $module     = $params->{module};
+   my $module     = $params->{module} or whimper 'No module name';
    my $distname   = $module; $distname =~ s{ :: }{-}gmx;
    my $class_path = catfile( q(lib), split m{ :: }mx, $module.q(.pm) );
 
@@ -37,7 +35,7 @@ sub new {
         create_packlist    => 0,
         create_readme      => 1,
         dist_version_from  => $class_path,
-        license            => $params->{license},
+        license            => $params->{license} || q(perl),
         meta_merge         => __get_resources( $params, $distname ),
         module_name        => $module,
         no_index           => __get_no_index( $params ),
@@ -61,7 +59,7 @@ sub __get_no_index {
 sub __get_notes {
    my $params = shift; my $notes = $params->{notes} || {};
 
-   $notes->{stop_tests} = $params->{stop_tests} && __cpan_testing()
+   $notes->{stop_tests} = ($params->{stop_tests} || 0) && __cpan_testing()
                         ? 'CPAN Testing stopped' : 0;
 
    return $notes;
